/**
   From
      http://www.ibm.com/developerworks/java/library/j-jtp11137/index.html
*/
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.RecursiveAction;
import java.util.Random;

class MaxWithFJ extends RecursiveAction
{
    private final int threshold;
    private final MaxProblem problem;
    public long result;


    public MaxWithFJ(MaxProblem problem, int threshold)
    {
        this.problem = problem;
        this.threshold = threshold;
    }


    @Override
    protected void compute()
    {
        if (problem.size < threshold)
            result = problem.solveSequentially();
        else
        {
            int midpoint = problem.size / 2;
            MaxWithFJ left =  new MaxWithFJ(problem.subproblem(0, midpoint), threshold);
            MaxWithFJ right = new MaxWithFJ(problem.subproblem(midpoint + 1, problem.size - 1), threshold);
            left.fork();      // asynchronous
            right.compute();  //  synchronous
            left.join();      //  synchronous
            result = Math.max(left.result, right.result);
        }
    }
}



class MaxProblem
{
   private final long[] numbers;
   private final int start;
   private final int end;
   public final int size;

   public MaxProblem(long[] numbers, int start, int end)
   {
      this.numbers = numbers;
      this.start = start;
      this.end = end;
      this.size = (end - start) + 1;
   }


   public long solveSequentially()
   {
      long max = Long.MIN_VALUE;
      for (int i = start; i <= end; i++)
      {
         long n = numbers[i];
         if (n > max) max = n;
      }
      return max;
   }


   public MaxProblem subproblem(int subStart, int subEnd)
   {
      return new MaxProblem(numbers, start + subStart, start + subEnd);
   }
}



public class Max_Java7
{
   public static final int PROBLEM_SIZE = 100_000_000;
   public static long numbers[] = new long[PROBLEM_SIZE];

   public static void main(String[] args)
   {
      Random generator = new Random();
      for (int i = 0; i < numbers.length; i++)
         numbers[i] = generator.nextLong();

      MaxProblem problem = new MaxProblem(numbers, 0, numbers.length - 1);

      long startTime;
      long stopTime;

      // Run the single-threaded sequential version.
      startTime = System.currentTimeMillis();
      long result = problem.solveSequentially();
      stopTime = System.currentTimeMillis();
      System.out.println("Sequential max: " + result);
      System.out.println("Sequential solution time: " + (stopTime - startTime));

      // Run the fork-join version.
      int threshold = 100;  // BE SURE TO EXPERIMENT WITH THIS!
      ForkJoinPool fjPool = new ForkJoinPool();
      MaxWithFJ mfj = new MaxWithFJ(problem, threshold);
      startTime = System.currentTimeMillis();
      fjPool.invoke(mfj);
      stopTime = System.currentTimeMillis();
      System.out.println(" Fork-join max: " + mfj.result);
      System.out.println(" Fork-join solution time: " + (stopTime - startTime));
      System.out.println("forkJoinPool: " + fjPool.toString());
   }
}