/**
   From
      http://www.ibm.com/developerworks/java/library/j-jtp03048/index.html
*/
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.RecursiveAction;
import java.util.Arrays;
import java.util.Random;

class MergeSort_Java7 extends RecursiveAction
{
   private final int SEQUENTIAL_THRESHOLD = 1_000;  // BE SURE TO EXPERIMENT WITH THIS!
   private final int[] numbers;
   private final int startPos;
   private final int endPos;
   public  final int[] result;


   public MergeSort_Java7(int[] numbers, int startPos, int endPos)
   {
      this.numbers = numbers;
      this.startPos = startPos;
      this.endPos = endPos;
      this.result = new int[numbers.length];
   }


   public int size()
   {
      return endPos - startPos;
   }


   private void merge(MergeSort_Java7 left, MergeSort_Java7 right)
   {
      int i=0, leftPos=0, rightPos=0, leftSize = left.size(), rightSize = right.size();

      while (leftPos < leftSize && rightPos < rightSize)
         result[i++] = (left.result[leftPos] <= right.result[rightPos])
                          ? left.result[leftPos++] : right.result[rightPos++];

      while (leftPos < leftSize)
         result[i++] = left.result[leftPos++];

      while (rightPos < rightSize)
         result[i++] = right.result[rightPos++];
   }//merge()


   protected void compute()
   {
      if (size() < SEQUENTIAL_THRESHOLD)
      {
         System.arraycopy(numbers, startPos, result, 0, size());
         Arrays.sort(result, 0, size());
      }
      else
      {
         int midpoint = size() / 2;
         MergeSort_Java7 left  = new MergeSort_Java7(numbers, startPos, startPos+midpoint);
         MergeSort_Java7 right = new MergeSort_Java7(numbers, startPos+midpoint, endPos);
         invokeAll(left, right);
         merge(left, right);
      }
   }//compute()


   public static void main(String[] args)
   {
      final int PROBLEM_SIZE = 1_000_000;

      // Create an array filled with random integers.
      int numbers[] = new int[PROBLEM_SIZE];
      Random generator = new Random();
      for (int i = 0; i < numbers.length; i++)
         numbers[i] = generator.nextInt();

      long startTime;
      long stopTime;

      // Run the fork-join version.
      int threshold = 100;  // BE SURE TO EXPERIMENT WITH THIS!
      ForkJoinPool forkJoinPool = new ForkJoinPool();
      MergeSort_Java7 task = new MergeSort_Java7(numbers, 0, numbers.length);
      startTime = System.currentTimeMillis();
      forkJoinPool.invoke(task);
      stopTime = System.currentTimeMillis();
      System.out.println("Fork-join solution time: " + (stopTime - startTime));
      System.out.println("forkJoinPool: " + forkJoinPool.toString());
      // print the first couple of hundred numbers in six COLUMNS.
      for (int i = 0; i < 30; i++)
      {
         for (int j = 0; j < 6; j++)
         {
            System.out.printf("%d ", task.result[i+(30*j)]);
         }
         System.out.printf("\n");
      }

      // Run a sequential version.
      startTime = System.currentTimeMillis();
      int[] result = new int[numbers.length];
      System.arraycopy(numbers, 0, result, 0, numbers.length);
      Arrays.sort(result, 0, result.length);
      stopTime = System.currentTimeMillis();
      System.out.println("Sequential solution time: " + (stopTime - startTime));
      // print the first couple of hundred numbers in six COLUMNS.
      for (int i = 0; i < 30; i++)
      {
         for (int j = 0; j < 6; j++)
         {
            System.out.printf("%d ", result[i+(30*j)]);
         }
         System.out.printf("\n");
      }
   }//main()
}//MergeSort_Java7