/**
   Search an array for the first occurrence of a number.
   Return the first index, if found, else return -1.
*/
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.RecursiveAction;
import java.util.Random;

class SearchWithFJ extends RecursiveAction
{
    private final int threshold;
    private final SearchProblem problem;
    public int result;


    public SearchWithFJ(SearchProblem problem, int threshold)
    {
        this.problem = problem;
        this.threshold = threshold;
    }


    @Override
    protected void compute()
    {
        if (problem.size < threshold)
            result = problem.solveSequentially();
        else
        {
            int midpoint = problem.size / 2;
            SearchWithFJ left =  new SearchWithFJ(problem.subproblem(0, midpoint), threshold);
            SearchWithFJ right = new SearchWithFJ(problem.subproblem(midpoint + 1, problem.size - 1), threshold);
            left.fork();
            right.compute();
            left.join();
            result = (0 <= left.result) ? left.result : right.result;
        }
    }
}



class SearchProblem
{
   private final int target;
   private final int[] numbers;
   private final int start;
   private final int end;
   public final int size;

   public SearchProblem(int target, int[] numbers, int start, int end)
   {
      this.target = target;
      this.numbers = numbers;
      this.start = start;
      this.end = end;
      this.size = (end - start) + 1;
   }


   public int solveSequentially()
   {
      int i = start;
      for (; (i <= end) && (numbers[i] != target); i++){}
      return (i == end+1) ? -1 : i;
   }


   public SearchProblem subproblem(int subStart, int subEnd)
   {
      return new SearchProblem(target, numbers, start + subStart, start + subEnd);
   }
}



public class Search_Java7
{
   public static final int PROBLEM_SIZE = 250_000_000;
   public static int numbers[] = new int[PROBLEM_SIZE];

   public static void main(String[] args)
   {
      Random generator = new Random();
      for (int i = 0; i < numbers.length; i++)
         numbers[i] = generator.nextInt() % (2 * PROBLEM_SIZE);
      // The probability of finding the following target is 0.5?
      int target = generator.nextInt() % (2 * PROBLEM_SIZE);

      SearchProblem problem = new SearchProblem(target, numbers, 0, numbers.length - 1);

      long startTime;
      long stopTime;

      // Run the single-threaded sequential version.
      startTime = System.currentTimeMillis();
      int result = problem.solveSequentially();
      stopTime = System.currentTimeMillis();
      System.out.println("Sequential search: " + result);
      System.out.println("Sequential solution time: " + (stopTime - startTime));

      // Run the fork-join version.
      int threshold = 1000;  // BE SURE TO EXPERIMENT WITH THIS!
      ForkJoinPool fjPool = new ForkJoinPool();
      SearchWithFJ mfj = new SearchWithFJ(problem, threshold);
      startTime = System.currentTimeMillis();
      fjPool.invoke(mfj);
      stopTime = System.currentTimeMillis();
      System.out.println(" Fork-join search: " + mfj.result);
      System.out.println(" Fork-join solution time: " + (stopTime - startTime));
      System.out.println("forkJoinPool: " + fjPool.toString());
   }
}