/**
   From
      http://www.ibm.com/developerworks/java/library/j-jtp11137/index.html
*/
import java.util.concurrent.ForkJoinPool;
import java.util.Random;

class MaxProblem implements Problem<Long>
{
   private final long[] numbers;
   private final int start;
   private final int end;
   public final int size;

   public MaxProblem(long[] numbers, int start, int end)
   {
      this.numbers = numbers;
      this.start = start;
      this.end = end;
      this.size = (end - start) + 1;
   }



   public int getSize()
   { return size;
   }

   public Long solveSequentially()
   {
      long max = Long.MIN_VALUE;
      for (int i = start; i <= end; i++)
      {
         long n = numbers[i];
         if (n > max) max = n;
      }
      return max;
   }

   public Long combine(Long left, Long right)
   {
      return Math.max( left, right );
   }

    public MaxProblem leftSubproblem()
   {
      return new MaxProblem(numbers, start, start + size/2 - 1);
   }

   public MaxProblem rightSubproblem()
   {
      return new MaxProblem(numbers, start + size/2, end);
   }
}


public class Max_Java7
{
   public static final int PROBLEM_SIZE = 10_000_000;
   public static long numbers[] = new long[PROBLEM_SIZE];

   public static void main(String[] args)
   {
      Random generator = new Random();
      for (int i = 0; i < numbers.length; i++)
         numbers[i] = generator.nextLong();



      MaxProblem problem = new MaxProblem(numbers, 0, numbers.length - 1);

      long startTime;
      long stopTime;

      // Run the single-threaded sequential version.
      startTime = System.currentTimeMillis();
      long result = problem.solveSequentially();
      stopTime = System.currentTimeMillis();
      System.out.println("Sequential solution: " + result);
      System.out.println("Sequential solution time: " + (stopTime - startTime));

      // Run the fork-join version.
      int threshold = 10_000;  // BE SURE TO EXPERIMENT WITH THIS!
      ForkJoinPool fjPool = new ForkJoinPool();
      ProblemWithFJ<Long> mfj = new ProblemWithFJ(problem, threshold);
      startTime = System.currentTimeMillis();
      fjPool.invoke(mfj);
      stopTime = System.currentTimeMillis();
      System.out.println(" Fork-join solution: " + mfj.result);
      System.out.println(" Fork-join solution time: " + (stopTime - startTime));
      System.out.println("forkJoinPool: " + fjPool.toString());
   }
}