/**
   Search an array for the first occurrence of a number.
   Return the first index, if found, else return -1.
*/
import java.util.concurrent.ForkJoinPool;
import java.util.Random;

class SearchProblem implements Problem<Integer>
{
   private final int target;
   private final int[] numbers;
   private final int start;
   private final int end;
   public final int size;

   public SearchProblem(int target, int[] numbers, int start, int end)
   {
      this.target = target;
      this.numbers = numbers;
      this.start = start;
      this.end = end;
      this.size = (end - start) + 1;
   }

   public int getSize()
   { return size;
   }

   public Integer solveSequentially()
   {
      int i = start;
      for (; (i <= end) && (numbers[i] != target); i++){}
      return (i == end+1) ? -1 : i;
   }

   public Integer combine(Integer left, Integer right)
   {
      return (0 <= left) ? left : right;
   }

   public SearchProblem leftSubproblem()
   {
      return new SearchProblem(target, numbers, start, start + size/2 - 1);
   }

   public SearchProblem rightSubproblem()
   {
      return new SearchProblem(target, numbers, start + size/2, end);
   }
}


public class Search_Java7
{
   public static final int PROBLEM_SIZE = 25_000_000;
   public static int numbers[] = new int[PROBLEM_SIZE];

   public static void main(String[] args)
   {
      Random generator = new Random();
      for (int i = 0; i < numbers.length; i++)
         numbers[i] = generator.nextInt() % (2 * PROBLEM_SIZE);
      // The probability of finding the following target is 0.5?
      int target = generator.nextInt() % (2 * PROBLEM_SIZE);

      SearchProblem problem = new SearchProblem(target, numbers, 0, numbers.length - 1);

      long startTime;
      long stopTime;

      // Run the single-threaded sequential version.
      startTime = System.currentTimeMillis();
      int result = problem.solveSequentially();
      stopTime = System.currentTimeMillis();
      System.out.println("Sequential solution: " + result);
      System.out.println("Sequential solution time: " + (stopTime - startTime));

      // Run the fork-join version.
      int threshold = 10_000;  // BE SURE TO EXPERIMENT WITH THIS!
      ForkJoinPool fjPool = new ForkJoinPool();
      ProblemWithFJ<Integer> mfj = new ProblemWithFJ(problem, threshold);
      startTime = System.currentTimeMillis();
      fjPool.invoke(mfj);
      stopTime = System.currentTimeMillis();
      System.out.println(" Fork-join solution: " + mfj.result);
      System.out.println(" Fork-join solution time: " + (stopTime - startTime));
      System.out.println("forkJoinPool: " + fjPool.toString());
   }
}