/**
   Search an array for the first occurrence of a number.
   Return the first index, if found, else return -1.
*/
import java.util.concurrent.ForkJoinPool;
import java.util.Random;

public class Search_Java7
{
   public static final int PROBLEM_SIZE = 25_000_000;
   public static int numbers[] = new int[PROBLEM_SIZE];

   public static void main(String[] args)
   {
      if (0 == args.length)
      {
         System.err.println("Usage: java Search_Java7 <size> <threshold>");
         System.exit(-1);
      }
      int size = Integer.parseInt(args[0]);
      int threshold = Integer.parseInt(args[1]);

      int numbers[] = new int[size];

      Random generator = new Random();
      for (int i = 0; i < numbers.length; i++)
         numbers[i] = generator.nextInt() % (2 * size);
      // The probability of finding the following target is 0.5?
      int target = generator.nextInt() % (2 * size);

      SearchProblem problem = new SearchProblem(target, numbers, 0, numbers.length - 1);

      long startTime;
      long stopTime;

      // Run the single-threaded sequential version.
      startTime = System.currentTimeMillis();
      int result = problem.solveSequentially(); // synchronous function call
      stopTime = System.currentTimeMillis();
      System.out.println("Sequential solution: " + result);
      System.out.println("Sequential solution time: " + (stopTime - startTime));

      // Run the fork-join version.
      ForkJoinPool fjPool = new ForkJoinPool();
      SolveProblemWithFJ<Integer> fj = new SolveProblemWithFJ<Integer>(problem, threshold);
      startTime = System.currentTimeMillis();
      fjPool.invoke(fj);        //  synchronous task
      stopTime = System.currentTimeMillis();
      System.out.println(" Fork-join solution: " + fj.result);
      System.out.println(" Fork-join solution time: " + (stopTime - startTime));
      System.out.println("forkJoinPool: " + fjPool.toString());
   }
}