import java.util.concurrent.RecursiveAction;

public class SolveProblemWithFJ<T> extends RecursiveAction
{
    private final int threshold;
    private final Problem<T> problem;
    public T result;


    public SolveProblemWithFJ(Problem<T> problem, int threshold)
    {
        this.problem = problem;
        this.threshold = threshold;
    }


    @Override
    protected void compute()
    {
        if (problem.getSize() < threshold)
            result = problem.solveSequentially();
        else
        {
            SolveProblemWithFJ<T> left  = new SolveProblemWithFJ<T>(problem.leftSubproblem(),  threshold);
            SolveProblemWithFJ<T> right = new SolveProblemWithFJ<T>(problem.rightSubproblem(), threshold);
            left.fork();      // asynchronous
            right.compute();  //  synchronous
            left.join();      //  synchronous
            result = problem.combine(left.result, right.result);
        }
    }
}