import java.util.concurrent.RecursiveAction;

public class SolveProblemWithFJ<T extends Problem<T>> extends RecursiveAction
{
    private final int threshold;
    private final T problem;

    public SolveProblemWithFJ(T problem, int threshold)
    {
        this.problem = problem;
        this.threshold = threshold;
    }

    @Override
    protected void compute()
    {
        if (problem.getSize() < threshold)
            problem.solveSequentially();
        else
        {
            SolveProblemWithFJ<T> left  = new SolveProblemWithFJ<T>(problem.leftSubproblem(),  threshold);
            SolveProblemWithFJ<T> right = new SolveProblemWithFJ<T>(problem.rightSubproblem(), threshold);
            left.fork();      // asynchronous
            right.compute();  //  synchronous
            left.join();      //  synchronous
            problem.combine(left.problem, right.problem);
        }
    }
}