import java.util.concurrent.RecursiveAction;

public abstract class Problem extends RecursiveAction
{
    protected final int threshold;

    protected Problem(int threshold)
    {
        this.threshold = threshold;
    }

    @Override
    protected void compute()
    {
        if (this.getSize() < threshold)
            this.solveSequentially();
        else
        {
            Problem left  = this.leftSubproblem();
            Problem right = this.rightSubproblem();
            left.fork();      // asynchronous
            right.compute();  //  synchronous
            left.join();      //  synchronous
            this.combine(left, right);
        }
    }

   public abstract int getSize();
   public abstract void solveSequentially();
   public abstract Problem leftSubproblem();
   public abstract Problem rightSubproblem();
   public abstract void combine(Problem left, Problem right);
}
